@@ -54,15 +54,13 @@ func (s *importsState) runProcessEnvFunc(ctx context.Context, snapshot *snapshot
 
 	// view.goEnv is immutable -- changes make a new view. Options can change.
 	// We can't compare build flags directly because we may add -modfile.
-	snapshot.view.optionsMu.Lock()
-	localPrefix := snapshot.view.options.Local
-	currentBuildFlags := snapshot.view.options.BuildFlags
-	currentDirectoryFilters := snapshot.view.options.DirectoryFilters
+	localPrefix := snapshot.Options().Local
+	currentBuildFlags := snapshot.Options().BuildFlags
+	currentDirectoryFilters := snapshot.Options().DirectoryFilters
 	changed := !reflect.DeepEqual(currentBuildFlags, s.cachedBuildFlags) ||
-		snapshot.view.options.VerboseOutput != (s.processEnv.Logf != nil) ||
+		snapshot.Options().VerboseOutput != (s.processEnv.Logf != nil) ||
 		modFileHash != s.cachedModFileHash ||
-		!reflect.DeepEqual(snapshot.view.options.DirectoryFilters, s.cachedDirectoryFilters)
-	snapshot.view.optionsMu.Unlock()
+		!reflect.DeepEqual(snapshot.Options().DirectoryFilters, s.cachedDirectoryFilters)
 
 	// If anything relevant to imports has changed, clear caches and
 	// update the processEnv. Clearing caches blocks on any background
@@ -120,7 +118,7 @@ func populateProcessEnvFromSnapshot(ctx context.Context, pe *imports.ProcessEnv,
 	ctx, done := event.Start(ctx, "cache.populateProcessEnvFromSnapshot")
 	defer done()
 
-	if snapshot.view.Options().VerboseOutput {
+	if snapshot.Options().VerboseOutput {
 		pe.Logf = func(format string, args ...interface{}) {
 			event.Log(ctx, fmt.Sprintf(format, args...))
 		}
@@ -135,7 +133,7 @@ func populateProcessEnvFromSnapshot(ctx context.Context, pe *imports.ProcessEnv,
 	// and has led to memory leaks in the past, when the snapshot was
 	// unintentionally held past its lifetime.
 	_, inv, cleanupInvocation, err := snapshot.goCommandInvocation(ctx, source.LoadWorkspace, &gocommand.Invocation{
-		WorkingDir: snapshot.view.workingDir().Filename(),
+		WorkingDir: snapshot.view.goCommandDir.Filename(),
 	})
 	if err != nil {
 		return err
@@ -154,7 +152,7 @@ func populateProcessEnvFromSnapshot(ctx context.Context, pe *imports.ProcessEnv,
 	// We don't actually use the invocation, so clean it up now.
 	cleanupInvocation()
 	// TODO(rfindley): should this simply be inv.WorkingDir?
-	pe.WorkingDir = snapshot.view.workingDir().Filename()
+	pe.WorkingDir = snapshot.view.goCommandDir.Filename()
 	return nil
 }
 
