@@ -11,11 +11,9 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path"
 	"path/filepath"
-	"reflect"
 	"regexp"
 	"sort"
 	"strings"
@@ -25,36 +23,41 @@ import (
 	"golang.org/x/mod/modfile"
 	"golang.org/x/mod/semver"
 	exec "golang.org/x/sys/execabs"
-	"golang.org/x/tools/gopls/internal/govulncheck"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/source"
 	"golang.org/x/tools/gopls/internal/span"
+	"golang.org/x/tools/gopls/internal/vulncheck"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/gocommand"
 	"golang.org/x/tools/internal/imports"
 	"golang.org/x/tools/internal/xcontext"
 )
 
+// A Folder represents an LSP workspace folder, together with its per-folder
+// options.
+//
+// Folders (Name and Dir) are specified by the 'initialize' and subsequent
+// 'didChangeWorkspaceFolders' requests; their options come from
+// didChangeConfiguration.
+//
+// Folders must not be mutated, as they may be shared across multiple views.
+type Folder struct {
+	Dir     span.URI
+	Name    string
+	Options *source.Options
+}
+
 type View struct {
 	id string
 
 	gocmdRunner *gocommand.Runner // limits go command concurrency
 
-	// baseCtx is the context handed to NewView. This is the parent of all
-	// background contexts created for this view.
-	baseCtx context.Context
-
-	// name is the user-specified name of this view.
-	name string
-
-	optionsMu sync.Mutex
-	options   *source.Options
+	folder *Folder
 
 	// Workspace information. The fields below are immutable, and together with
 	// options define the build list. Any change to these fields results in a new
 	// View.
-	folder               span.URI // user-specified workspace folder
-	workspaceInformation          // Go environment information
+	*workspaceInformation // Go environment information
 
 	importsState *importsState
 
@@ -65,7 +68,7 @@ type View struct {
 
 	// vulns maps each go.mod file's URI to its known vulnerabilities.
 	vulnsMu sync.Mutex
-	vulns   map[span.URI]*govulncheck.Result
+	vulns   map[span.URI]*vulncheck.Result
 
 	// parseCache holds an LRU cache of recently parsed files.
 	parseCache *parseCache
@@ -73,7 +76,7 @@ type View struct {
 	// fs is the file source used to populate this view.
 	fs *overlayFS
 
-	// seenFiles tracks files that the view has accessed.
+	// knownFiles tracks files that the view has accessed.
 	// TODO(golang/go#57558): this notion is fundamentally problematic, and
 	// should be removed.
 	knownFilesMu sync.Mutex
@@ -137,6 +140,13 @@ type workspaceInformation struct {
 	// inGOPATH reports whether the workspace directory is contained in a GOPATH
 	// directory.
 	inGOPATH bool
+
+	// goCommandDir is the dir to use for running go commands.
+	//
+	// The only case where this should matter is if we've narrowed the workspace to
+	// a single nested module. In that case, the go command won't be able to find
+	// the module unless we tell it the nested directory.
+	goCommandDir span.URI
 }
 
 // effectiveGO111MODULE reports the value of GO111MODULE effective in the go
@@ -351,7 +361,7 @@ func (v *View) ID() string { return v.id }
 // longer needed.
 func tempModFile(modFh source.FileHandle, gosum []byte) (tmpURI span.URI, cleanup func(), err error) {
 	filenameHash := source.Hashf("%s", modFh.URI().Filename())
-	tmpMod, err := ioutil.TempFile("", fmt.Sprintf("go.%s.*.mod", filenameHash))
+	tmpMod, err := os.CreateTemp("", fmt.Sprintf("go.%s.*.mod", filenameHash))
 	if err != nil {
 		return "", nil, err
 	}
@@ -386,7 +396,7 @@ func tempModFile(modFh source.FileHandle, gosum []byte) (tmpURI span.URI, cleanu
 
 	// Create an analogous go.sum, if one exists.
 	if gosum != nil {
-		if err := ioutil.WriteFile(tmpSumName, gosum, 0655); err != nil {
+		if err := os.WriteFile(tmpSumName, gosum, 0655); err != nil {
 			return "", nil, err
 		}
 	}
@@ -396,102 +406,45 @@ func tempModFile(modFh source.FileHandle, gosum []byte) (tmpURI span.URI, cleanu
 
 // Name returns the user visible name of this view.
 func (v *View) Name() string {
-	return v.name
+	return v.folder.Name
 }
 
 // Folder returns the folder at the base of this view.
 func (v *View) Folder() span.URI {
-	return v.folder
+	return v.folder.Dir
 }
 
-func (v *View) Options() *source.Options {
-	v.optionsMu.Lock()
-	defer v.optionsMu.Unlock()
-	return v.options
-}
+// SetFolderOptions updates the options of each View associated with the folder
+// of the given URI.
+//
+// Calling this may cause each related view to be invalidated and a replacement
+// view added to the session.
+func (s *Session) SetFolderOptions(ctx context.Context, uri span.URI, options *source.Options) error {
+	s.viewMu.Lock()
+	defer s.viewMu.Unlock()
 
-func (v *View) FileKind(fh source.FileHandle) source.FileKind {
-	// The kind of an unsaved buffer comes from the
-	// TextDocumentItem.LanguageID field in the didChange event,
-	// not from the file name. They may differ.
-	if o, ok := fh.(*Overlay); ok {
-		if o.kind != source.UnknownKind {
-			return o.kind
+	for _, v := range s.views {
+		if v.folder.Dir == uri {
+			folder2 := *v.folder
+			folder2.Options = options
+			info, err := getWorkspaceInformation(ctx, s.gocmdRunner, s, &folder2)
+			if err != nil {
+				return err
+			}
+			if _, err := s.updateViewLocked(ctx, v, info, &folder2); err != nil {
+				return err
+			}
 		}
 	}
-
-	fext := filepath.Ext(fh.URI().Filename())
-	switch fext {
-	case ".go":
-		return source.Go
-	case ".mod":
-		return source.Mod
-	case ".sum":
-		return source.Sum
-	case ".work":
-		return source.Work
-	}
-	exts := v.Options().TemplateExtensions
-	for _, ext := range exts {
-		if fext == ext || fext == "."+ext {
-			return source.Tmpl
-		}
-	}
-	// and now what? This should never happen, but it does for cgo before go1.15
-	return source.Go
-}
-
-func minorOptionsChange(a, b *source.Options) bool {
-	// Check if any of the settings that modify our understanding of files have
-	// been changed.
-	if !reflect.DeepEqual(a.Env, b.Env) {
-		return false
-	}
-	if !reflect.DeepEqual(a.DirectoryFilters, b.DirectoryFilters) {
-		return false
-	}
-	if !reflect.DeepEqual(a.StandaloneTags, b.StandaloneTags) {
-		return false
-	}
-	if a.ExpandWorkspaceToModule != b.ExpandWorkspaceToModule {
-		return false
-	}
-	if a.MemoryMode != b.MemoryMode {
-		return false
-	}
-	aBuildFlags := make([]string, len(a.BuildFlags))
-	bBuildFlags := make([]string, len(b.BuildFlags))
-	copy(aBuildFlags, a.BuildFlags)
-	copy(bBuildFlags, b.BuildFlags)
-	sort.Strings(aBuildFlags)
-	sort.Strings(bBuildFlags)
-	// the rest of the options are benign
-	return reflect.DeepEqual(aBuildFlags, bBuildFlags)
-}
-
-// SetViewOptions sets the options of the given view to new values. Calling
-// this may cause the view to be invalidated and a replacement view added to
-// the session. If so the new view will be returned, otherwise the original one
-// will be returned.
-func (s *Session) SetViewOptions(ctx context.Context, v *View, options *source.Options) (*View, error) {
-	// no need to rebuild the view if the options were not materially changed
-	v.optionsMu.Lock()
-	if minorOptionsChange(v.options, options) {
-		v.options = options
-		v.optionsMu.Unlock()
-		return v, nil
-	}
-	v.optionsMu.Unlock()
-	newView, err := s.updateView(ctx, v, options)
-	return newView, err
+	return nil
 }
 
 // viewEnv returns a string describing the environment of a newly created view.
 //
 // It must not be called concurrently with any other view methods.
 func viewEnv(v *View) string {
-	env := v.options.EnvSlice()
-	buildFlags := append([]string{}, v.options.BuildFlags...)
+	env := v.folder.Options.EnvSlice()
+	buildFlags := append([]string{}, v.folder.Options.BuildFlags...)
 
 	var buf bytes.Buffer
 	fmt.Fprintf(&buf, `go info for %v
@@ -501,8 +454,8 @@ func viewEnv(v *View) string {
 (build flags: %v)
 (selected go env: %v)
 `,
-		v.folder.Filename(),
-		v.workingDir().Filename(),
+		v.folder.Dir.Filename(),
+		v.goCommandDir.Filename(),
 		strings.TrimRight(v.workspaceInformation.goversionOutput, "\n"),
 		v.snapshot.validBuildConfiguration(),
 		buildFlags,
@@ -540,14 +493,14 @@ func fileHasExtension(path string, suffixes []string) bool {
 // locateTemplateFiles ensures that the snapshot has mapped template files
 // within the workspace folder.
 func (s *snapshot) locateTemplateFiles(ctx context.Context) {
-	if len(s.view.Options().TemplateExtensions) == 0 {
+	suffixes := s.Options().TemplateExtensions
+	if len(suffixes) == 0 {
 		return
 	}
-	suffixes := s.view.Options().TemplateExtensions
 
 	searched := 0
 	filterFunc := s.view.filterFunc()
-	err := filepath.WalkDir(s.view.folder.Filename(), func(path string, entry os.DirEntry, err error) error {
+	err := filepath.WalkDir(s.view.folder.Dir.Filename(), func(path string, entry os.DirEntry, err error) error {
 		if err != nil {
 			return err
 		}
@@ -587,10 +540,10 @@ func (v *View) contains(uri span.URI) bool {
 	// user. It would be better to explicitly consider the set of active modules
 	// wherever relevant.
 	inGoDir := false
-	if source.InDir(v.workingDir().Filename(), v.folder.Filename()) {
-		inGoDir = source.InDir(v.workingDir().Filename(), uri.Filename())
+	if source.InDir(v.goCommandDir.Filename(), v.folder.Dir.Filename()) {
+		inGoDir = source.InDir(v.goCommandDir.Filename(), uri.Filename())
 	}
-	inFolder := source.InDir(v.folder.Filename(), uri.Filename())
+	inFolder := source.InDir(v.folder.Dir.Filename(), uri.Filename())
 
 	if !inGoDir && !inFolder {
 		return false
@@ -602,11 +555,12 @@ func (v *View) contains(uri span.URI) bool {
 // filterFunc returns a func that reports whether uri is filtered by the currently configured
 // directoryFilters.
 func (v *View) filterFunc() func(span.URI) bool {
-	filterer := buildFilterer(v.folder.Filename(), v.gomodcache, v.Options())
+	folderDir := v.folder.Dir.Filename()
+	filterer := buildFilterer(folderDir, v.gomodcache, v.folder.Options)
 	return func(uri span.URI) bool {
 		// Only filter relative to the configured root directory.
-		if source.InDir(v.folder.Filename(), uri.Filename()) {
-			return pathExcludedByFilter(strings.TrimPrefix(uri.Filename(), v.folder.Filename()), filterer)
+		if source.InDir(folderDir, uri.Filename()) {
+			return pathExcludedByFilter(strings.TrimPrefix(uri.Filename(), folderDir), filterer)
 		}
 		return false
 	}
@@ -779,7 +733,6 @@ func (s *snapshot) initialize(ctx context.Context, firstAttempt bool) {
 	}
 
 	s.loadWorkspace(ctx, firstAttempt)
-	s.collectAllKnownSubdirs(ctx)
 }
 
 func (s *snapshot) loadWorkspace(ctx context.Context, firstAttempt bool) (loadErr error) {
@@ -912,7 +865,9 @@ func (s *snapshot) loadWorkspace(ctx context.Context, firstAttempt bool) (loadEr
 //
 // invalidateContent returns a non-nil snapshot for the new content, along with
 // a callback which the caller must invoke to release that snapshot.
-func (v *View) invalidateContent(ctx context.Context, changes map[span.URI]*fileChange, forceReloadMetadata bool) (*snapshot, func()) {
+//
+// newOptions may be nil, in which case options remain unchanged.
+func (v *View) invalidateContent(ctx context.Context, changes map[span.URI]source.FileHandle) (*snapshot, func()) {
 	// Detach the context so that content invalidation cannot be canceled.
 	ctx = xcontext.Detach(ctx)
 
@@ -934,7 +889,7 @@ func (v *View) invalidateContent(ctx context.Context, changes map[span.URI]*file
 	prevSnapshot.AwaitInitialized(ctx)
 
 	// Save one lease of the cloned snapshot in the view.
-	v.snapshot, v.releaseSnapshot = prevSnapshot.clone(ctx, v.baseCtx, changes, forceReloadMetadata)
+	v.snapshot, v.releaseSnapshot = prevSnapshot.clone(ctx, changes)
 
 	prevReleaseSnapshot()
 	v.destroy(prevSnapshot, "View.invalidateContent")
@@ -943,25 +898,25 @@ func (v *View) invalidateContent(ctx context.Context, changes map[span.URI]*file
 	return v.snapshot, v.snapshot.Acquire()
 }
 
-func (s *Session) getWorkspaceInformation(ctx context.Context, folder span.URI, options *source.Options) (workspaceInformation, error) {
-	if err := checkPathCase(folder.Filename()); err != nil {
-		return workspaceInformation{}, fmt.Errorf("invalid workspace folder path: %w; check that the casing of the configured workspace folder path agrees with the casing reported by the operating system", err)
+func getWorkspaceInformation(ctx context.Context, runner *gocommand.Runner, fs source.FileSource, folder *Folder) (*workspaceInformation, error) {
+	if err := checkPathCase(folder.Dir.Filename()); err != nil {
+		return nil, fmt.Errorf("invalid workspace folder path: %w; check that the casing of the configured workspace folder path agrees with the casing reported by the operating system", err)
 	}
+	info := new(workspaceInformation)
 	var err error
-	var info workspaceInformation
 	inv := gocommand.Invocation{
-		WorkingDir: folder.Filename(),
-		Env:        options.EnvSlice(),
+		WorkingDir: folder.Dir.Filename(),
+		Env:        folder.Options.EnvSlice(),
 	}
-	info.goversion, err = gocommand.GoVersion(ctx, inv, s.gocmdRunner)
+	info.goversion, err = gocommand.GoVersion(ctx, inv, runner)
 	if err != nil {
 		return info, err
 	}
-	info.goversionOutput, err = gocommand.GoVersionOutput(ctx, inv, s.gocmdRunner)
+	info.goversionOutput, err = gocommand.GoVersionOutput(ctx, inv, runner)
 	if err != nil {
 		return info, err
 	}
-	if err := info.load(ctx, folder.Filename(), options.EnvSlice(), s.gocmdRunner); err != nil {
+	if err := info.load(ctx, folder.Dir.Filename(), folder.Options.EnvSlice(), runner); err != nil {
 		return info, err
 	}
 	// The value of GOPACKAGESDRIVER is not returned through the go command.
@@ -973,19 +928,33 @@ func (s *Session) getWorkspaceInformation(ctx context.Context, folder span.URI,
 
 	// filterFunc is the path filter function for this workspace folder. Notably,
 	// it is relative to folder (which is specified by the user), not root.
-	filterFunc := pathExcludedByFilterFunc(folder.Filename(), info.gomodcache, options)
-	info.gomod, err = findWorkspaceModFile(ctx, folder, s, filterFunc)
+	filterFunc := pathExcludedByFilterFunc(folder.Dir.Filename(), info.gomodcache, folder.Options)
+	info.gomod, err = findWorkspaceModFile(ctx, folder.Dir, fs, filterFunc)
 	if err != nil {
 		return info, err
 	}
 
 	// Check if the workspace is within any GOPATH directory.
 	for _, gp := range filepath.SplitList(info.gopath) {
-		if source.InDir(filepath.Join(gp, "src"), folder.Filename()) {
+		if source.InDir(filepath.Join(gp, "src"), folder.Dir.Filename()) {
 			info.inGOPATH = true
 			break
 		}
 	}
+
+	// Compute the "working directory", which is where we run go commands.
+	//
+	// Note: if gowork is in use, this will default to the workspace folder. In
+	// the past, we would instead use the folder containing go.work. This should
+	// not make a difference, and in fact may improve go list error messages.
+	//
+	// TODO(golang/go#57514): eliminate the expandWorkspaceToModule setting
+	// entirely.
+	if folder.Options.ExpandWorkspaceToModule && info.gomod != "" {
+		info.goCommandDir = span.URIFromPath(filepath.Dir(info.gomod.Filename()))
+	} else {
+		info.goCommandDir = folder.Dir
+	}
 	return info, nil
 }
 
@@ -1027,24 +996,6 @@ func findWorkspaceModFile(ctx context.Context, folderURI span.URI, fs source.Fil
 	return "", nil
 }
 
-// workingDir returns the directory from which to run Go commands.
-//
-// The only case where this should matter is if we've narrowed the workspace to
-// a singular nested module. In that case, the go command won't be able to find
-// the module unless we tell it the nested directory.
-func (v *View) workingDir() span.URI {
-	// Note: if gowork is in use, this will default to the workspace folder. In
-	// the past, we would instead use the folder containing go.work. This should
-	// not make a difference, and in fact may improve go list error messages.
-	//
-	// TODO(golang/go#57514): eliminate the expandWorkspaceToModule setting
-	// entirely.
-	if v.Options().ExpandWorkspaceToModule && v.gomod != "" {
-		return span.URIFromPath(filepath.Dir(v.gomod.Filename()))
-	}
-	return v.folder
-}
-
 // findRootPattern looks for files with the given basename in dir or any parent
 // directory of dir, using the provided FileSource. It returns the first match,
 // starting from dir and search parents.
@@ -1054,11 +1005,11 @@ func (v *View) workingDir() span.URI {
 func findRootPattern(ctx context.Context, dir, basename string, fs source.FileSource) (string, error) {
 	for dir != "" {
 		target := filepath.Join(dir, basename)
-		exists, err := fileExists(ctx, span.URIFromPath(target), fs)
+		fh, err := fs.ReadFile(ctx, span.URIFromPath(target))
 		if err != nil {
-			return "", err // not readable or context cancelled
+			return "", err // context cancelled
 		}
-		if exists {
+		if fileExists(fh) {
 			return target, nil
 		}
 		// Trailing separators must be trimmed, otherwise filepath.Split is a noop.
@@ -1122,8 +1073,8 @@ func (v *View) ClearModuleUpgrades(modfile span.URI) {
 const maxGovulncheckResultAge = 1 * time.Hour // Invalidate results older than this limit.
 var timeNow = time.Now                        // for testing
 
-func (v *View) Vulnerabilities(modfiles ...span.URI) map[span.URI]*govulncheck.Result {
-	m := make(map[span.URI]*govulncheck.Result)
+func (v *View) Vulnerabilities(modfiles ...span.URI) map[span.URI]*vulncheck.Result {
+	m := make(map[span.URI]*vulncheck.Result)
 	now := timeNow()
 	v.vulnsMu.Lock()
 	defer v.vulnsMu.Unlock()
@@ -1144,7 +1095,7 @@ func (v *View) Vulnerabilities(modfiles ...span.URI) map[span.URI]*govulncheck.R
 	return m
 }
 
-func (v *View) SetVulnerabilities(modfile span.URI, vulns *govulncheck.Result) {
+func (v *View) SetVulnerabilities(modfile span.URI, vulns *vulncheck.Result) {
 	v.vulnsMu.Lock()
 	defer v.vulnsMu.Unlock()
 
@@ -1233,7 +1184,7 @@ func (s *snapshot) vendorEnabled(ctx context.Context, modURI span.URI, modConten
 	// No vendor directory?
 	// TODO(golang/go#57514): this is wrong if the working dir is not the module
 	// root.
-	if fi, err := os.Stat(filepath.Join(s.view.workingDir().Filename(), "vendor")); err != nil || !fi.IsDir() {
+	if fi, err := os.Stat(filepath.Join(s.view.goCommandDir.Filename(), "vendor")); err != nil || !fi.IsDir() {
 		return false, nil
 	}
 
