@@ -5,7 +5,6 @@
 package debug
 
 import (
-	"archive/zip"
 	"bytes"
 	"context"
 	"errors"
@@ -20,16 +19,15 @@ import (
 	"path"
 	"path/filepath"
 	"runtime"
-	rpprof "runtime/pprof"
 	"strconv"
 	"strings"
 	"sync"
 	"time"
 
+	"golang.org/x/tools/gopls/internal/bug"
 	"golang.org/x/tools/gopls/internal/lsp/cache"
 	"golang.org/x/tools/gopls/internal/lsp/debug/log"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
-	"golang.org/x/tools/internal/bug"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/event/core"
 	"golang.org/x/tools/internal/event/export"
@@ -107,6 +105,13 @@ func (st *State) Cache(id string) *cache.Cache {
 	return nil
 }
 
+// Analysis returns the global Analysis template value.
+func (st *State) Analysis() (_ analysisTmpl) { return }
+
+type analysisTmpl struct{}
+
+func (analysisTmpl) AnalyzerRunTimes() []cache.LabelDuration { return cache.AnalyzerRunTimes() }
+
 // Sessions returns the set of Session objects currently being served.
 func (st *State) Sessions() []*cache.Session {
 	var sessions []*cache.Session
@@ -192,14 +197,14 @@ type Server struct {
 	ClientID     string
 }
 
-// AddClient adds a client to the set being served.
+// addClient adds a client to the set being served.
 func (st *State) addClient(session *cache.Session) {
 	st.mu.Lock()
 	defer st.mu.Unlock()
 	st.clients = append(st.clients, &Client{Session: session})
 }
 
-// DropClient removes a client from the set being served.
+// dropClient removes a client from the set being served.
 func (st *State) dropClient(session *cache.Session) {
 	st.mu.Lock()
 	defer st.mu.Unlock()
@@ -213,7 +218,7 @@ func (st *State) dropClient(session *cache.Session) {
 	}
 }
 
-// AddServer adds a server to the set being queried. In practice, there should
+// updateServer updates a server to the set being queried. In practice, there should
 // be at most one remote server.
 func (st *State) updateServer(server *Server) {
 	st.mu.Lock()
@@ -232,7 +237,7 @@ func (st *State) updateServer(server *Server) {
 	st.servers = append(st.servers, server)
 }
 
-// DropServer drops a server from the set being queried.
+// dropServer drops a server from the set being queried.
 func (st *State) dropServer(id string) {
 	st.mu.Lock()
 	defer st.mu.Unlock()
@@ -280,6 +285,10 @@ func (i *Instance) getCache(r *http.Request) interface{} {
 	return i.State.Cache(path.Base(r.URL.Path))
 }
 
+func (i *Instance) getAnalysis(r *http.Request) interface{} {
+	return i.State.Analysis()
+}
+
 func (i *Instance) getSession(r *http.Request) interface{} {
 	return i.State.Session(path.Base(r.URL.Path))
 }
@@ -452,6 +461,7 @@ func (i *Instance) Serve(ctx context.Context, addr string) (string, error) {
 		if i.traces != nil {
 			mux.HandleFunc("/trace/", render(TraceTmpl, i.traces.getData))
 		}
+		mux.HandleFunc("/analysis/", render(AnalysisTmpl, i.getAnalysis))
 		mux.HandleFunc("/cache/", render(CacheTmpl, i.getCache))
 		mux.HandleFunc("/session/", render(SessionTmpl, i.getSession))
 		mux.HandleFunc("/view/", render(ViewTmpl, i.getView))
@@ -461,8 +471,15 @@ func (i *Instance) Serve(ctx context.Context, addr string) (string, error) {
 		mux.HandleFunc("/info", render(InfoTmpl, i.getInfo))
 		mux.HandleFunc("/memory", render(MemoryTmpl, getMemory))
 
+		// Internal debugging helpers.
+		mux.HandleFunc("/gc", func(w http.ResponseWriter, r *http.Request) {
+			runtime.GC()
+			runtime.GC()
+			runtime.GC()
+			http.Redirect(w, r, "/memory", http.StatusTemporaryRedirect)
+		})
 		mux.HandleFunc("/_makeabug", func(w http.ResponseWriter, r *http.Request) {
-			bug.Report("bug here", nil)
+			bug.Report("bug here")
 			http.Error(w, "made a bug", http.StatusOK)
 		})
 
@@ -487,75 +504,6 @@ func (i *Instance) ListenedDebugAddress() string {
 	return i.listenedDebugAddress
 }
 
-// MonitorMemory starts recording memory statistics each second.
-func (i *Instance) MonitorMemory(ctx context.Context) {
-	tick := time.NewTicker(time.Second)
-	nextThresholdGiB := uint64(1)
-	go func() {
-		for {
-			<-tick.C
-			var mem runtime.MemStats
-			runtime.ReadMemStats(&mem)
-			if mem.HeapAlloc < nextThresholdGiB*1<<30 {
-				continue
-			}
-			if err := i.writeMemoryDebug(nextThresholdGiB, true); err != nil {
-				event.Error(ctx, "writing memory debug info", err)
-			}
-			if err := i.writeMemoryDebug(nextThresholdGiB, false); err != nil {
-				event.Error(ctx, "writing memory debug info", err)
-			}
-			event.Log(ctx, fmt.Sprintf("Wrote memory usage debug info to %v", os.TempDir()))
-			nextThresholdGiB++
-		}
-	}()
-}
-
-func (i *Instance) writeMemoryDebug(threshold uint64, withNames bool) error {
-	suffix := "withnames"
-	if !withNames {
-		suffix = "nonames"
-	}
-
-	filename := fmt.Sprintf("gopls.%d-%dGiB-%s.zip", os.Getpid(), threshold, suffix)
-	zipf, err := os.OpenFile(filepath.Join(os.TempDir(), filename), os.O_CREATE|os.O_RDWR, 0644)
-	if err != nil {
-		return err
-	}
-	zipw := zip.NewWriter(zipf)
-
-	f, err := zipw.Create("heap.pb.gz")
-	if err != nil {
-		return err
-	}
-	if err := rpprof.Lookup("heap").WriteTo(f, 0); err != nil {
-		return err
-	}
-
-	f, err = zipw.Create("goroutines.txt")
-	if err != nil {
-		return err
-	}
-	if err := rpprof.Lookup("goroutine").WriteTo(f, 1); err != nil {
-		return err
-	}
-
-	for _, cache := range i.State.Caches() {
-		cf, err := zipw.Create(fmt.Sprintf("cache-%v.html", cache.ID()))
-		if err != nil {
-			return err
-		}
-		if _, err := cf.Write([]byte(cache.PackageStats(withNames))); err != nil {
-			return err
-		}
-	}
-
-	if err := zipw.Close(); err != nil {
-		return err
-	}
-	return zipf.Close()
-}
-
 func makeGlobalExporter(stderr io.Writer) event.Exporter {
 	p := export.Printer{}
 	var pMu sync.Mutex
@@ -700,10 +648,10 @@ var BaseTemplate = template.Must(template.New("").Parse(`
 td.value {
   text-align: right;
 }
-ul.events {
-	list-style-type: none;
+ul.spans {
+	font-family: monospace;
+	font-size:   85%;
 }
-
 </style>
 {{block "head" .}}{{end}}
 </head>
@@ -711,9 +659,11 @@ ul.events {
 <a href="/">Main</a>
 <a href="/info">Info</a>
 <a href="/memory">Memory</a>
+<a href="/debug/pprof">Profiling</a>
 <a href="/metrics">Metrics</a>
 <a href="/rpc">RPC</a>
 <a href="/trace">Trace</a>
+<a href="/analysis">Analysis</a>
 <hr>
 <h1>{{template "title" .}}</h1>
 {{block "body" .}}
@@ -727,7 +677,6 @@ Unknown page
 {{define "serverlink"}}<a href="/server/{{.}}">Server {{.}}</a>{{end}}
 {{define "sessionlink"}}<a href="/session/{{.}}">Session {{.}}</a>{{end}}
 {{define "viewlink"}}<a href="/view/{{.}}">View {{.}}</a>{{end}}
-{{define "filelink"}}<a href="/file/{{.Session}}/{{.FileIdentity.Hash}}">{{.FileIdentity.URI}}</a>{{end}}
 `)).Funcs(template.FuncMap{
 	"fuint64":  fuint64,
 	"fuint32":  fuint32,
@@ -752,9 +701,10 @@ Unknown page
 		}
 		return s
 	},
-	"options": func(s *cache.Session) []sessionOption {
-		return showOptions(s.Options())
-	},
+	// TODO(rfindley): re-enable option inspection.
+	// "options": func(s *cache.Session) []sessionOption {
+	// 	return showOptions(s.Options())
+	// },
 })
 
 var MainTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
@@ -781,9 +731,10 @@ var InfoTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
 `))
 
 var MemoryTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
-{{define "title"}}GoPls memory usage{{end}}
+{{define "title"}}Gopls memory usage{{end}}
 {{define "head"}}<meta http-equiv="refresh" content="5">{{end}}
 {{define "body"}}
+<form action="/gc"><input type="submit" value="Run garbage collector"/></form>
 <h2>Stats</h2>
 <table>
 <tr><td class="label">Allocated bytes</td><td class="value">{{fuint64 .HeapAlloc}}</td></tr>
@@ -822,8 +773,14 @@ var CacheTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
 {{define "body"}}
 <h2>memoize.Store entries</h2>
 <ul>{{range $k,$v := .MemStats}}<li>{{$k}} - {{$v}}</li>{{end}}</ul>
-<h2>Per-package usage - not accurate, for guidance only</h2>
-{{.PackageStats true}}
+{{end}}
+`))
+
+var AnalysisTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
+{{define "title"}}Analysis{{end}}
+{{define "body"}}
+<h2>Analyzer.Run times</h2>
+<ul>{{range .AnalyzerRunTimes}}<li>{{.Duration}} {{.Label}}</li>{{end}}</ul>
 {{end}}
 `))
 
@@ -876,13 +833,11 @@ From: <b>{{template "cachelink" .Cache.ID}}</b><br>
 <h2>Views</h2>
 <ul>{{range .Views}}<li>{{.Name}} is {{template "viewlink" .ID}} in {{.Folder}}</li>{{end}}</ul>
 <h2>Overlays</h2>
-<ul>{{range .Overlays}}<li>{{template "filelink" .}}</li>{{end}}</ul>
-<h2>Options</h2>
-{{range options .}}
-<p><b>{{.Name}}</b> {{.Type}}</p>
-<p><i>default:</i> {{.Default}}</p>
-{{if ne .Default .Current}}<p><i>current:</i> {{.Current}}</p>{{end}}
-{{end}}
+{{$session := .}}
+<ul>{{range .Overlays}}
+<li>
+<a href="/file/{{$session.ID}}/{{.FileIdentity.Hash}}">{{.FileIdentity.URI}}</a>
+</li>{{end}}</ul>
 {{end}}
 `))
 
@@ -891,8 +846,6 @@ var ViewTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
 {{define "body"}}
 Name: <b>{{.Name}}</b><br>
 Folder: <b>{{.Folder}}</b><br>
-<h2>Environment</h2>
-<ul>{{range .Options.Env}}<li>{{.}}</li>{{end}}</ul>
 {{end}}
 `))
 
@@ -900,13 +853,12 @@ var FileTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
 {{define "title"}}Overlay {{.FileIdentity.Hash}}{{end}}
 {{define "body"}}
 {{with .}}
-	From: <b>{{template "sessionlink" .Session}}</b><br>
 	URI: <b>{{.URI}}</b><br>
 	Identifier: <b>{{.FileIdentity.Hash}}</b><br>
 	Version: <b>{{.Version}}</b><br>
 	Kind: <b>{{.Kind}}</b><br>
 {{end}}
 <h3>Contents</h3>
-<pre>{{fcontent .Read}}</pre>
+<pre>{{fcontent .Content}}</pre>
 {{end}}
 `))
