@@ -11,11 +11,11 @@ import (
 	"strings"
 	"testing"
 
+	"golang.org/x/tools/gopls/internal/bug"
 	"golang.org/x/tools/gopls/internal/hooks"
 	"golang.org/x/tools/gopls/internal/lsp"
 	"golang.org/x/tools/gopls/internal/lsp/fake"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
-	"golang.org/x/tools/internal/bug"
 	"golang.org/x/tools/internal/gocommand"
 	"golang.org/x/tools/internal/testenv"
 
@@ -131,7 +131,7 @@ func TestReferences(t *testing.T) {
 			WithOptions(opts...).Run(t, workspaceModule, func(t *testing.T, env *Env) {
 				f := "pkg/inner/inner.go"
 				env.OpenFile(f)
-				locations := env.References(f, env.RegexpSearch(f, `SaySomething`))
+				locations := env.References(env.RegexpSearch(f, `SaySomething`))
 				want := 3
 				if got := len(locations); got != want {
 					t.Fatalf("expected %v locations, got %v", want, got)
@@ -151,12 +151,12 @@ func TestClearAnalysisDiagnostics(t *testing.T) {
 		WorkspaceFolders("pkg/inner"),
 	).Run(t, workspaceModule, func(t *testing.T, env *Env) {
 		env.OpenFile("pkg/main.go")
-		env.Await(
-			env.DiagnosticAtRegexp("pkg/main2.go", "fmt.Print"),
+		env.AfterChange(
+			Diagnostics(env.AtRegexp("pkg/main2.go", "fmt.Print")),
 		)
 		env.CloseBuffer("pkg/main.go")
-		env.Await(
-			EmptyDiagnostics("pkg/main2.go"),
+		env.AfterChange(
+			NoDiagnostics(ForFile("pkg/main2.go")),
 		)
 	})
 }
@@ -178,34 +178,7 @@ replace random.org => %s
 `, env.ReadWorkspaceFile("pkg/go.mod"), dir)
 		env.WriteWorkspaceFile("pkg/go.mod", goModWithReplace)
 		env.Await(
-			OnceMet(
-				env.DoneWithChangeWatchedFiles(),
-				LogMatching(protocol.Info, `packages\.Load #\d+\n`, 2, false),
-			),
-		)
-	})
-}
-
-// This test checks that gopls updates the set of files it watches when a
-// replace target is added to the go.mod.
-func TestWatchReplaceTargets(t *testing.T) {
-	t.Skipf("skipping known-flaky test: see https://go.dev/issue/50748")
-
-	WithOptions(
-		ProxyFiles(workspaceProxy),
-		WorkspaceFolders("pkg"),
-	).Run(t, workspaceModule, func(t *testing.T, env *Env) {
-		// Add a replace directive and expect the files that gopls is watching
-		// to change.
-		dir := env.Sandbox.Workdir.URI("goodbye").SpanURI().Filename()
-		goModWithReplace := fmt.Sprintf(`%s
-replace random.org => %s
-`, env.ReadWorkspaceFile("pkg/go.mod"), dir)
-		env.WriteWorkspaceFile("pkg/go.mod", goModWithReplace)
-		env.Await(
-			env.DoneWithChangeWatchedFiles(),
-			UnregistrationMatching("didChangeWatchedFiles"),
-			RegistrationMatching("didChangeWatchedFiles"),
+			LogMatching(protocol.Info, `packages\.Load #\d+\n`, 2, false),
 		)
 	})
 }
@@ -270,9 +243,9 @@ func Hello() int {
 		env.RunGoCommand("work", "init")
 		env.RunGoCommand("work", "use", "-r", ".")
 		env.AfterChange(
-			env.DiagnosticAtRegexp("moda/a/a.go", "x"),
-			env.DiagnosticAtRegexp("modb/b/b.go", "x"),
-			env.NoDiagnosticAtRegexp("moda/a/a.go", `"b.com/b"`),
+			Diagnostics(env.AtRegexp("moda/a/a.go", "x")),
+			Diagnostics(env.AtRegexp("modb/b/b.go", "x")),
+			NoDiagnostics(env.AtRegexp("moda/a/a.go", `"b.com/b"`)),
 		)
 	})
 }
@@ -326,8 +299,9 @@ func main() {
 	WithOptions(
 		ProxyFiles(proxy),
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
-		env.Await(
-			env.DiagnosticAtRegexp("main.go", "x"),
+		env.OnceMet(
+			InitialWorkspaceLoad,
+			Diagnostics(env.AtRegexp("main.go", "x")),
 		)
 	})
 }
@@ -381,7 +355,8 @@ func Hello() int {
 		env.OpenFile("moda/a/a.go")
 		env.Await(env.DoneWithOpen())
 
-		original, _ := env.GoToDefinition("moda/a/a.go", env.RegexpSearch("moda/a/a.go", "Hello"))
+		originalLoc := env.GoToDefinition(env.RegexpSearch("moda/a/a.go", "Hello"))
+		original := env.Sandbox.Workdir.URIToPath(originalLoc.URI)
 		if want := "modb/b/b.go"; !strings.HasSuffix(original, want) {
 			t.Errorf("expected %s, got %v", want, original)
 		}
@@ -393,7 +368,8 @@ func Hello() int {
 		env.WriteWorkspaceFile("go.work", "go 1.18\nuse moda/a")
 		env.AfterChange()
 
-		got, _ := env.GoToDefinition("moda/a/a.go", env.RegexpSearch("moda/a/a.go", "Hello"))
+		gotLoc := env.GoToDefinition(env.RegexpSearch("moda/a/a.go", "Hello"))
+		got := env.Sandbox.Workdir.URIToPath(gotLoc.URI)
 		if want := "b.com@v1.2.3/b/b.go"; !strings.HasSuffix(got, want) {
 			t.Errorf("expected %s, got %v", want, got)
 		}
@@ -434,7 +410,8 @@ func main() {
 		ProxyFiles(workspaceModuleProxy),
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
 		env.OpenFile("moda/a/a.go")
-		original, _ := env.GoToDefinition("moda/a/a.go", env.RegexpSearch("moda/a/a.go", "Hello"))
+		loc := env.GoToDefinition(env.RegexpSearch("moda/a/a.go", "Hello"))
+		original := env.Sandbox.Workdir.URIToPath(loc.URI)
 		if want := "b.com@v1.2.3/b/b.go"; !strings.HasSuffix(original, want) {
 			t.Errorf("expected %s, got %v", want, original)
 		}
@@ -455,8 +432,9 @@ func Hello() int {
 }
 `,
 		})
-		env.AfterChange(env.DiagnosticAtRegexp("modb/b/b.go", "x"))
-		got, _ := env.GoToDefinition("moda/a/a.go", env.RegexpSearch("moda/a/a.go", "Hello"))
+		env.AfterChange(Diagnostics(env.AtRegexp("modb/b/b.go", "x")))
+		gotLoc := env.GoToDefinition(env.RegexpSearch("moda/a/a.go", "Hello"))
+		got := env.Sandbox.Workdir.URIToPath(gotLoc.URI)
 		if want := "modb/b/b.go"; !strings.HasSuffix(got, want) {
 			t.Errorf("expected %s, got %v", want, original)
 		}
@@ -466,8 +444,6 @@ func Hello() int {
 // This test confirms that a gopls workspace can recover from initialization
 // with one invalid module.
 func TestOneBrokenModule(t *testing.T) {
-	t.Skip("golang/go#55331: this test is temporarily broken as go.work handling tries to build the workspace module")
-
 	testenv.NeedsGo1Point(t, 18) // uses go.work
 	const multiModule = `
 -- go.work --
@@ -507,149 +483,17 @@ func Hello() int {
 		ProxyFiles(workspaceModuleProxy),
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
 		env.OpenFile("modb/go.mod")
-		env.Await(
-			OnceMet(
-				env.DoneWithOpen(),
-				DiagnosticAt("modb/go.mod", 0, 0),
-			),
+		env.AfterChange(
+			Diagnostics(AtPosition("modb/go.mod", 0, 0)),
 		)
 		env.RegexpReplace("modb/go.mod", "modul", "module")
 		env.SaveBufferWithoutActions("modb/go.mod")
-		env.Await(
-			env.DiagnosticAtRegexp("modb/b/b.go", "x"),
+		env.AfterChange(
+			Diagnostics(env.AtRegexp("modb/b/b.go", "x")),
 		)
 	})
 }
 
-func TestUseGoplsMod(t *testing.T) {
-	// This test validates certain functionality related to using a gopls.mod
-	// file to specify workspace modules.
-	const multiModule = `
--- moda/a/go.mod --
-module a.com
-
-require b.com v1.2.3
--- moda/a/go.sum --
-b.com v1.2.3 h1:tXrlXP0rnjRpKNmkbLYoWBdq0ikb3C3bKK9//moAWBI=
-b.com v1.2.3/go.mod h1:D+J7pfFBZK5vdIdZEFquR586vKKIkqG7Qjw9AxG5BQ8=
--- moda/a/a.go --
-package a
-
-import (
-	"b.com/b"
-)
-
-func main() {
-	var x int
-	_ = b.Hello()
-}
--- modb/go.mod --
-module b.com
-
-require example.com v1.2.3
--- modb/go.sum --
-example.com v1.2.3 h1:veRD4tUnatQRgsULqULZPjeoBGFr2qBhevSCZllD2Ds=
-example.com v1.2.3/go.mod h1:Y2Rc5rVWjWur0h3pd9aEvK5Pof8YKDANh9gHA2Maujo=
--- modb/b/b.go --
-package b
-
-func Hello() int {
-	var x int
-}
--- gopls.mod --
-module gopls-workspace
-
-require (
-	a.com v0.0.0-goplsworkspace
-	b.com v1.2.3
-)
-
-replace a.com => $SANDBOX_WORKDIR/moda/a
-`
-	WithOptions(
-		ProxyFiles(workspaceModuleProxy),
-		Modes(Experimental),
-	).Run(t, multiModule, func(t *testing.T, env *Env) {
-		// Initially, the gopls.mod should cause only the a.com module to be
-		// loaded. Validate this by jumping to a definition in b.com and ensuring
-		// that we go to the module cache.
-		env.OpenFile("moda/a/a.go")
-		env.Await(env.DoneWithOpen())
-
-		// To verify which modules are loaded, we'll jump to the definition of
-		// b.Hello.
-		checkHelloLocation := func(want string) error {
-			location, _ := env.GoToDefinition("moda/a/a.go", env.RegexpSearch("moda/a/a.go", "Hello"))
-			if !strings.HasSuffix(location, want) {
-				return fmt.Errorf("expected %s, got %v", want, location)
-			}
-			return nil
-		}
-
-		// Initially this should be in the module cache, as b.com is not replaced.
-		if err := checkHelloLocation("b.com@v1.2.3/b/b.go"); err != nil {
-			t.Fatal(err)
-		}
-
-		// Now, modify the gopls.mod file on disk to activate the b.com module in
-		// the workspace.
-		workdir := env.Sandbox.Workdir.RootURI().SpanURI().Filename()
-		env.WriteWorkspaceFile("gopls.mod", fmt.Sprintf(`module gopls-workspace
-
-require (
-	a.com v1.9999999.0-goplsworkspace
-	b.com v1.9999999.0-goplsworkspace
-)
-
-replace a.com => %s/moda/a
-replace b.com => %s/modb
-`, workdir, workdir))
-
-		// As of golang/go#54069, writing a gopls.mod to the workspace triggers a
-		// workspace reload.
-		env.Await(
-			OnceMet(
-				env.DoneWithChangeWatchedFiles(),
-				env.DiagnosticAtRegexp("modb/b/b.go", "x"),
-			),
-		)
-
-		// Jumping to definition should now go to b.com in the workspace.
-		if err := checkHelloLocation("modb/b/b.go"); err != nil {
-			t.Fatal(err)
-		}
-
-		// Now, let's modify the gopls.mod *overlay* (not on disk), and verify that
-		// this change is only picked up once it is saved.
-		env.OpenFile("gopls.mod")
-		env.Await(env.DoneWithOpen())
-		env.SetBufferContent("gopls.mod", fmt.Sprintf(`module gopls-workspace
-
-require (
-	a.com v0.0.0-goplsworkspace
-)
-
-replace a.com => %s/moda/a
-`, workdir))
-
-		// Editing the gopls.mod removes modb from the workspace modules, and so
-		// should clear outstanding diagnostics...
-		env.Await(OnceMet(
-			env.DoneWithChange(),
-			EmptyDiagnostics("modb/go.mod"),
-		))
-		// ...but does not yet cause a workspace reload, so we should still jump to modb.
-		if err := checkHelloLocation("modb/b/b.go"); err != nil {
-			t.Fatal(err)
-		}
-		// Saving should reload the workspace.
-		env.SaveBufferWithoutActions("gopls.mod")
-		if err := checkHelloLocation("b.com@v1.2.3/b/b.go"); err != nil {
-			t.Fatal(err)
-		}
-	})
-}
-
 // TestBadGoWork exercises the panic from golang/vscode-go#2121.
 func TestBadGoWork(t *testing.T) {
 	const files = `
@@ -664,6 +508,7 @@ module example.com/bar
 }
 
 func TestUseGoWork(t *testing.T) {
+	testenv.NeedsGo1Point(t, 18) // uses go.work
 	// This test validates certain functionality related to using a go.work
 	// file to specify workspace modules.
 	const multiModule = `
@@ -707,19 +552,28 @@ use (
 `
 	WithOptions(
 		ProxyFiles(workspaceModuleProxy),
+		Settings{
+			"subdirWatchPatterns": "on",
+		},
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
-		// Initially, the go.work should cause only the a.com module to be
-		// loaded. Validate this by jumping to a definition in b.com and ensuring
-		// that we go to the module cache.
+		// Initially, the go.work should cause only the a.com module to be loaded,
+		// so we shouldn't get any file watches for modb. Further validate this by
+		// jumping to a definition in b.com and ensuring that we go to the module
+		// cache.
+		env.OnceMet(
+			InitialWorkspaceLoad,
+			NoFileWatchMatching("modb"),
+		)
 		env.OpenFile("moda/a/a.go")
 		env.Await(env.DoneWithOpen())
 
 		// To verify which modules are loaded, we'll jump to the definition of
 		// b.Hello.
 		checkHelloLocation := func(want string) error {
-			location, _ := env.GoToDefinition("moda/a/a.go", env.RegexpSearch("moda/a/a.go", "Hello"))
-			if !strings.HasSuffix(location, want) {
-				return fmt.Errorf("expected %s, got %v", want, location)
+			loc := env.GoToDefinition(env.RegexpSearch("moda/a/a.go", "Hello"))
+			file := env.Sandbox.Workdir.URIToPath(loc.URI)
+			if !strings.HasSuffix(file, want) {
+				return fmt.Errorf("expected %s, got %v", want, file)
 			}
 			return nil
 		}
@@ -741,12 +595,13 @@ use (
 `)
 
 		// As of golang/go#54069, writing go.work to the workspace triggers a
-		// workspace reload.
-		env.Await(
-			OnceMet(
-				env.DoneWithChangeWatchedFiles(),
-				env.DiagnosticAtRegexp("modb/b/b.go", "x"),
-			),
+		// workspace reload, and new file watches.
+		env.AfterChange(
+			Diagnostics(env.AtRegexp("modb/b/b.go", "x")),
+			// TODO(golang/go#60340): we don't get a file watch yet, because
+			// updateWatchedDirectories runs before snapshot.load. Instead, we get it
+			// after the next change (the didOpen below).
+			// FileWatchMatching("modb"),
 		)
 
 		// Jumping to definition should now go to b.com in the workspace.
@@ -757,7 +612,13 @@ use (
 		// Now, let's modify the go.work *overlay* (not on disk), and verify that
 		// this change is only picked up once it is saved.
 		env.OpenFile("go.work")
-		env.Await(env.DoneWithOpen())
+		env.AfterChange(
+			// TODO(golang/go#60340): delete this expectation in favor of
+			// the commented-out expectation above, once we fix the evaluation order
+			// of file watches. We should not have to wait for a second change to get
+			// the correct watches.
+			FileWatchMatching("modb"),
+		)
 		env.SetBufferContent("go.work", `go 1.17
 
 use (
@@ -781,8 +642,8 @@ use (
 		}
 
 		// This fails if guarded with a OnceMet(DoneWithSave(), ...), because it is
-		// debounced (and therefore not synchronous with the change).
-		env.Await(EmptyOrNoDiagnostics("modb/go.mod"))
+		// delayed (and therefore not synchronous with the change).
+		env.Await(NoDiagnostics(ForFile("modb/go.mod")))
 
 		// Test Formatting.
 		env.SetBufferContent("go.work", `go 1.18
@@ -809,6 +670,8 @@ use (
 }
 
 func TestUseGoWorkDiagnosticMissingModule(t *testing.T) {
+	testenv.NeedsGo1Point(t, 18) // uses go.work
+
 	const files = `
 -- go.work --
 go 1.18
@@ -819,8 +682,8 @@ module example.com/bar
 `
 	Run(t, files, func(t *testing.T, env *Env) {
 		env.OpenFile("go.work")
-		env.Await(
-			env.DiagnosticAtRegexpWithMessage("go.work", "use", "directory ./foo does not contain a module"),
+		env.AfterChange(
+			Diagnostics(env.AtRegexp("go.work", "use"), WithMessage("directory ./foo does not contain a module")),
 		)
 		// The following tests is a regression test against an issue where we weren't
 		// copying the workFile struct field on workspace when a new one was created in
@@ -829,17 +692,18 @@ module example.com/bar
 		// struct, and then set the content back to the old contents to make sure
 		// the diagnostic still shows up.
 		env.SetBufferContent("go.work", "go 1.18 \n\n use ./bar\n")
-		env.Await(
-			env.NoDiagnosticAtRegexp("go.work", "use"),
+		env.AfterChange(
+			NoDiagnostics(env.AtRegexp("go.work", "use")),
 		)
 		env.SetBufferContent("go.work", "go 1.18 \n\n use ./foo\n")
-		env.Await(
-			env.DiagnosticAtRegexpWithMessage("go.work", "use", "directory ./foo does not contain a module"),
+		env.AfterChange(
+			Diagnostics(env.AtRegexp("go.work", "use"), WithMessage("directory ./foo does not contain a module")),
 		)
 	})
 }
 
 func TestUseGoWorkDiagnosticSyntaxError(t *testing.T) {
+	testenv.NeedsGo1Point(t, 18)
 	const files = `
 -- go.work --
 go 1.18
@@ -849,14 +713,16 @@ replace
 `
 	Run(t, files, func(t *testing.T, env *Env) {
 		env.OpenFile("go.work")
-		env.Await(
-			env.DiagnosticAtRegexpWithMessage("go.work", "usa", "unknown directive: usa"),
-			env.DiagnosticAtRegexpWithMessage("go.work", "replace", "usage: replace"),
+		env.AfterChange(
+			Diagnostics(env.AtRegexp("go.work", "usa"), WithMessage("unknown directive: usa")),
+			Diagnostics(env.AtRegexp("go.work", "replace"), WithMessage("usage: replace")),
 		)
 	})
 }
 
 func TestUseGoWorkHover(t *testing.T) {
+	testenv.NeedsGo1Point(t, 18)
+
 	const files = `
 -- go.work --
 go 1.18
@@ -883,8 +749,7 @@ module example.com/bar/baz
 		}
 
 		for hoverRE, want := range tcs {
-			pos := env.RegexpSearch("go.work", hoverRE)
-			got, _ := env.Hover("go.work", pos)
+			got, _ := env.Hover(env.RegexpSearch("go.work", hoverRE))
 			if got.Value != want {
 				t.Errorf(`hover on %q: got %q, want %q`, hoverRE, got, want)
 			}
@@ -933,21 +798,22 @@ use (
 	).Run(t, workspace, func(t *testing.T, env *Env) {
 		env.OpenFile("moda/a/a.go")
 		env.Await(env.DoneWithOpen())
-		location, _ := env.GoToDefinition("moda/a/a.go", env.RegexpSearch("moda/a/a.go", "Hello"))
+		loc := env.GoToDefinition(env.RegexpSearch("moda/a/a.go", "Hello"))
+		file := env.Sandbox.Workdir.URIToPath(loc.URI)
 		want := "modb/b/b.go"
-		if !strings.HasSuffix(location, want) {
-			t.Errorf("expected %s, got %v", want, location)
+		if !strings.HasSuffix(file, want) {
+			t.Errorf("expected %s, got %v", want, file)
 		}
 	})
 }
 
 func TestNonWorkspaceFileCreation(t *testing.T) {
 	const files = `
--- go.mod --
+-- work/go.mod --
 module mod.com
 
 go 1.12
--- x.go --
+-- work/x.go --
 package x
 `
 
@@ -956,10 +822,12 @@ package foo
 import "fmt"
 var _ = fmt.Printf
 `
-	Run(t, files, func(t *testing.T, env *Env) {
-		env.CreateBuffer("/tmp/foo.go", "")
-		env.EditBuffer("/tmp/foo.go", fake.NewEdit(0, 0, 0, 0, code))
-		env.GoToDefinition("/tmp/foo.go", env.RegexpSearch("/tmp/foo.go", `Printf`))
+	WithOptions(
+		WorkspaceFolders("work"), // so that outside/... is outside the workspace
+	).Run(t, files, func(t *testing.T, env *Env) {
+		env.CreateBuffer("outside/foo.go", "")
+		env.EditBuffer("outside/foo.go", fake.NewEdit(0, 0, 0, 0, code))
+		env.GoToDefinition(env.RegexpSearch("outside/foo.go", `Printf`))
 	})
 }
 
@@ -1035,16 +903,14 @@ func main() {
 	WithOptions(
 		ProxyFiles(proxy),
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
-		env.Await(
-			OnceMet(
-				InitialWorkspaceLoad,
-				// TODO(rfindley): assert on the full set of diagnostics here. We
-				// should ensure that we don't have a diagnostic at b.Hi in a.go.
-				env.DiagnosticAtRegexp("moda/a/a.go", "x"),
-				env.DiagnosticAtRegexp("modb/b/b.go", "x"),
-				env.DiagnosticAtRegexp("modb/v2/b/b.go", "x"),
-				env.DiagnosticAtRegexp("modc/main.go", "x"),
-			),
+		env.OnceMet(
+			InitialWorkspaceLoad,
+			// TODO(rfindley): assert on the full set of diagnostics here. We
+			// should ensure that we don't have a diagnostic at b.Hi in a.go.
+			Diagnostics(env.AtRegexp("moda/a/a.go", "x")),
+			Diagnostics(env.AtRegexp("modb/b/b.go", "x")),
+			Diagnostics(env.AtRegexp("modb/v2/b/b.go", "x")),
+			Diagnostics(env.AtRegexp("modc/main.go", "x")),
 		)
 	})
 }
@@ -1096,11 +962,12 @@ func main() {
 	).Run(t, files, func(t *testing.T, env *Env) {
 		params := &protocol.PublishDiagnosticsParams{}
 		env.OpenFile("b/go.mod")
-		env.Await(
-			OnceMet(
-				env.GoSumDiagnostic("b/go.mod", `example.com v1.2.3`),
-				ReadDiagnostics("b/go.mod", params),
+		env.AfterChange(
+			Diagnostics(
+				env.AtRegexp("go.mod", `example.com v1.2.3`),
+				WithMessage("go.sum is out of sync"),
 			),
+			ReadDiagnostics("b/go.mod", params),
 		)
 		for _, d := range params.Diagnostics {
 			if !strings.Contains(d.Message, "go.sum is out of sync") {
@@ -1112,8 +979,8 @@ func main() {
 			}
 			env.ApplyQuickFixes("b/go.mod", []protocol.Diagnostic{d})
 		}
-		env.Await(
-			EmptyDiagnostics("b/go.mod"),
+		env.AfterChange(
+			NoDiagnostics(ForFile("b/go.mod")),
 		)
 	})
 }
@@ -1155,6 +1022,7 @@ package main
 }
 
 func TestAddAndRemoveGoWork(t *testing.T) {
+	testenv.NeedsGo1Point(t, 18)
 	// Use a workspace with a module in the root directory to exercise the case
 	// where a go.work is added to the existing root directory. This verifies
 	// that we're detecting changes to the module source, not just the root
@@ -1185,8 +1053,8 @@ func main() {}
 		// Since b/main.go is not in the workspace, it should have a warning on its
 		// package declaration.
 		env.AfterChange(
-			EmptyDiagnostics("main.go"),
-			DiagnosticAt("b/main.go", 0, 0),
+			NoDiagnostics(ForFile("main.go")),
+			Diagnostics(env.AtRegexp("b/main.go", "package (main)")),
 		)
 		env.WriteWorkspaceFile("go.work", `go 1.16
 
@@ -1195,7 +1063,7 @@ use (
 	b
 )
 `)
-		env.AfterChange(NoOutstandingDiagnostics())
+		env.AfterChange(NoDiagnostics())
 		// Removing the go.work file should put us back where we started.
 		env.RemoveWorkspaceFile("go.work")
 
@@ -1211,8 +1079,8 @@ use (
 		env.OpenFile("b/main.go")
 
 		env.AfterChange(
-			EmptyDiagnostics("main.go"),
-			DiagnosticAt("b/main.go", 0, 0),
+			NoDiagnostics(ForFile("main.go")),
+			Diagnostics(env.AtRegexp("b/main.go", "package (main)")),
 		)
 	})
 }
@@ -1248,17 +1116,17 @@ func (Server) Foo() {}
 		// as invalid. So we need to wait for the metadata of main_test.go to be
 		// updated before moving other_test.go back to the main_test package.
 		env.Await(
-			env.DiagnosticAtRegexp("other_test.go", "Server"),
-			env.DiagnosticAtRegexp("main_test.go", "otherConst"),
+			Diagnostics(env.AtRegexp("other_test.go", "Server")),
+			Diagnostics(env.AtRegexp("main_test.go", "otherConst")),
 		)
 		env.RegexpReplace("other_test.go", "main", "main_test")
-		env.Await(
-			EmptyDiagnostics("other_test.go"),
-			EmptyDiagnostics("main_test.go"),
+		env.AfterChange(
+			NoDiagnostics(ForFile("other_test.go")),
+			NoDiagnostics(ForFile("main_test.go")),
 		)
 
 		// This will cause a test failure if other_test.go is not in any package.
-		_, _ = env.GoToDefinition("other_test.go", env.RegexpSearch("other_test.go", "Server"))
+		_ = env.GoToDefinition(env.RegexpSearch("other_test.go", "Server"))
 	})
 }
 
@@ -1296,18 +1164,12 @@ import (
 `
 	Run(t, ws, func(t *testing.T, env *Env) {
 		env.OpenFile("b/main.go")
-		env.Await(
-			OnceMet(
-				env.DoneWithOpen(),
-				NoDiagnostics("a/main.go"),
-			),
+		env.AfterChange(
+			NoDiagnostics(ForFile("a/main.go")),
 		)
 		env.OpenFile("a/main.go")
-		env.Await(
-			OnceMet(
-				env.DoneWithOpen(),
-				env.DiagnosticAtRegexpWithMessage("a/main.go", "V", "not used"),
-			),
+		env.AfterChange(
+			Diagnostics(env.AtRegexp("a/main.go", "V"), WithMessage("not used")),
 		)
 		env.CloseBuffer("a/main.go")
 
@@ -1317,11 +1179,8 @@ import (
 		// TODO(rfindley): it should not be necessary to make another edit here.
 		// Gopls should be smart enough to avoid diagnosing a.
 		env.RegexpReplace("b/main.go", "package b", "package b // a package")
-		env.Await(
-			OnceMet(
-				env.DoneWithChange(),
-				EmptyDiagnostics("a/main.go"),
-			),
+		env.AfterChange(
+			NoDiagnostics(ForFile("a/main.go")),
 		)
 	})
 }
@@ -1335,11 +1194,9 @@ func TestOldGoNotification_SupportedVersion(t *testing.T) {
 	}
 
 	Run(t, "", func(t *testing.T, env *Env) {
-		env.Await(
-			OnceMet(
-				InitialWorkspaceLoad,
-				NoShownMessage("upgrade"),
-			),
+		env.OnceMet(
+			InitialWorkspaceLoad,
+			NoShownMessage("upgrade"),
 		)
 	})
 }
